/*******************************************************************************
 * Copyright (c) 2009, 2013 Oracle and/or its affiliates. All rights reserved.
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License v1.0 and Eclipse Distribution License v. 1.0
 * which accompanies this distribution.
 * The Eclipse Public License is available at http://www.eclipse.org/legal/epl-v10.html
 * and the Eclipse Distribution License is available at
 * http://www.eclipse.org/org/documents/edl-v10.php.
 *
 * Contributors:
 *     Oracle - initial API and implementation
 *
 ******************************************************************************/
package org.eclipse.persistence.tools.utility.iterator;

import java.util.Iterator;
import org.eclipse.persistence.tools.utility.ObjectTools;
import org.eclipse.persistence.tools.utility.collection.Queue;

/**
 * A <code>QueueIterator</code> provides an {@link Iterator}
 * for a {@link Queue} of objects of type <code>E</code>. The queue's elements
 * are {@link Queue#dequeue() dequeued}" as the iterator returns them with
 * calls to {@link #next()}.
 *
 * @param <E> the type of elements returned by the iterator
 *
 * @see Queue
 * @see org.eclipse.jpt.common.utility.internal.iterable.QueueIterable
 */
public class QueueIterator<E>
	implements Iterator<E>
{
	private final Queue<E> queue;


	/**
	 * Construct an iterator for the specified queue.
	 */
	public QueueIterator(Queue<E> queue) {
		super();
		if (queue == null) {
			throw new NullPointerException();
		}
		this.queue = queue;
	}

	@Override
	public boolean hasNext() {
		return ! this.queue.isEmpty();
	}

	@Override
	public E next() {
		return this.queue.dequeue();
	}

	@Override
	public void remove() {
		throw new UnsupportedOperationException();
	}

	@Override
	public String toString() {
		return ObjectTools.toString(this, this.queue);
	}
}